# <%= appname %>

> Generated by [nodena-api-php][npm-url]

## Information

## Runtime

*   [Git][Git]
*   [Node.js][Node.js]
    *   [Gulp][Gulp]
*   [PHP][PHP]
    *   [Composer][Composer]
*   Optional Node Packages
    *   [ESLint][ESLint]
*   Optional PHP Packages
    *   [PHP_CodeSniffer][PHP_CodeSniffer]

## File Structure

| File/Directory              | Description                                                 |
| --------------------------- | ----------------------------------------------------------- |
| `gulp-tasks/`               | [Gulp][Gulp] task definitions                               |
| `src/`                      | Source files                                                |
| `res/`                      | Resources, including docs, sql, etc.                        |
| `res/changelog.template.md` | Markdown template for changelog                             |
| `dist/`                     | Temporary build files for developement/debug                |
| `online/`                   | Files for online sites                                      |
| `online/stage/`             | Files for stage site                                        |
| `online/live/`              | Files for live site                                         |
| `online/patches/`           | Patch files for each version                                |
| `node_modules/`             | Dependent packages for Node.js                              |
| `vendor/`                   | Dependent packages for [Composer][Composer]                 |
| `.editorconfig`             | Editor config file for editors                              |
| `.eslintrc`                 | Setup file for [ESlint][ESlint]                             |
| `.gitignore`                | Ignored files list for [Git][Git]                           |
| `gulpfile.js`               | Project main workflow                                       |
| `gulptasks.js`              | Custom task loader for [gulp-task-loader][gulp-task-loader] |
| `package.json`              | Project meta information                                    |
| `composer.json`             | [Composer][Composer] dependency information                 |
| `README.md`                 | This readme file                                            |
| `CHANGELOG.md`              | Auto generated change log                                   |

## Environment Setup (`config/*`)

See [documents of `node-config`][node-config] for more information.

### root

Default: `api/`

Path to project root, it'll be created if not exist.

Example:

```json
{
  "root": "backend/"
}
```

### globals

Default: `null`

Define constant variables.

Options:

*   `php` - Variables will be injected by [`gulp-preprocess`][gulp-preprocess] to all PHP files.

Example:

```json
{
  "globals": {
    "php": {
      "dbConnect": {
        "dsn": "localhost",
        "username": "root",
        "password": "root"
      }
    }
  }
}
```

In PHP files:

```php
define('DB_DSN', '/* @echo dbConnect.dsn */');
```

### preprocess

Default: `null`

Extra settings for [gulp-preprocess][gulp-preprocess].

Options:

*   `filter` - Assign a filter, using [gulp-filter][gulp-filter].
    *   `php` - Filter on source files.

Example:

```json
{
  "preprocess": {
    "filter": {
      "php": ["**/*.php"]
    }
  }
}
```

### localServer

Setup for launch a local server.

#### browserSync

Options for [BrowserSync][BrowserSync].

Options:

*   `startPath` - Start path of server. Default: `config.root`.
*   `server.baseDir` - Base directory of server. Default: `config.assets.build`.
*   `server.index` - Default page. Default: `index.html`.
*   `https` - Enable self-signed SSL. Default: `false`.

See [other options of BrowserSync][browser-sync#options].

Example:

```javascript
module.exports = {
  localServer: {
    browserSync: {
      server: {
        index: 'index.php',
      },
      https: true,
    },
  }
};
```

#### php

Setup a [PHP server proxy][gulp-connect-php] for [BrowserSync][BrowserSync]

Options:

*   `base` - Base directory of server. Default: `config.assets.build`.

See [other options of PHP proxy][gulp-connect-php#options].

#### proxy

Assign proxies as middleware of [BrowserSync][BrowserSync].

Options:

*   `uri` - The URI path matching for proxy.
*   `options` - The proxy options. See [options of proxy][node-http-proxy#options].

Example:

```javascript
module.exports = {
  localServer: {
    proxy: [{
      uri: '/api',
      options: {
        target: 'https://mydomain.com',
        changeOrigin: true,
        secure: false,
      },
    }, {
      uri: '/upload',
      options: {
        target: 'http://mydomain.com',
        changeOrigin: true,
      },
    }],
  },
};
```

All requests on `/api` will proxy to `https://mydomain.com/api`,
and `/upload` will proxy to `https://mydomain.com/upload`

## Gulp Tasks

### Build

#### Build for Developement

Build and open live preview with [BrowserSync][BrowserSync].

```bash
NODE_ENV=development gulp
```

#### Build for Stage

Build a distribution for stage, all files will generated in ```dist/stage/```.

```bash
NODE_ENV=stage gulp
```

#### Build for Production

Build a distribution for production, all files will generated in ```dist/production/```

```bash
NODE_ENV=production gulp
```

#### Build Options

##### Verbose (default: false)

Disable all compression and show detailed debug information on console.

```bash
gulp -b
```

##### Port for BrowserSync (default: 8888)

Manually assign the port of live preview.

```bash
gulp -p 8889
```

### Helper

#### Bump New Version

Bump new [semver][semver] version number to `package.json` and create a git tag.

##### Prerelease => 1.0.x-stage.x

```bash
gulp version --new-version prerelease --preid stage
```

##### Patch => 1.0.x

```bash
gulp version --new-version patch
```

##### Minor => 1.x.0

```bash
gulp version --new-version minor
```

##### Major => x.0.0

```bash
gulp version --new-version major
```

##### Manually

```bash
gulp version --new-version 1.2.3
```

## Change Log

[Changelog](CHANGELOG.md)

[BrowserSync]: https://www.browsersync.io/
[Composer]: https://getcomposer.org/
[ESLint]: http://eslint.org/
[Git]: https://git-scm.com/
[Gulp]: http://gulpjs.com/
[Node.js]: https://nodejs.org/
[PHP]: http://php.net/
[PHP_CodeSniffer]: http://pear.php.net/package/PHP_CodeSniffer/
[browser-sync#options]: https://browsersync.io/docs/options
[gulp-connect-php]: https://github.com/micahblu/gulp-connect-php
[gulp-connect-php#options]: https://github.com/micahblu/gulp-connect-php#options
[gulp-filter]: https://github.com/sindresorhus/gulp-filter/
[gulp-preprocess]: https://github.com/jas/gulp-preprocess/
[gulp-task-loader]: https://github.com/hontas/gulp-task-loader/
[node-config]: https://github.com/lorenwest/node-config/
[node-http-proxy#options]: https://github.com/nodejitsu/node-http-proxy#options
[npm-url]: https://npmjs.org/package/generator-nodena-api-php/
[semver]: https://github.com/npm/node-semver/
